#!/usr/bin/python
# Copyright 2016, 2019 Odin Kroeger
#
# This programme is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This programme is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Setup for the *pandoc-refheadstyle* package."""

from os import path
from setuptools import setup



# Functions
# =========

def readme(readme_fname="README.rst"):
    """Returns the contents of README.rst.

    :param str readme_fname: Path to README.rst.
    :returns: Contents of *readme_fname*.
    :rtype: str
    """
    readme_path = path.join(path.dirname(__file__), readme_fname)
    with open(readme_path) as readme_handle:
        return readme_handle.read()


# Metadata
# ========

# Name of this package.
NAME = 'pandoc-refheadstyle'

# Version of this package.
VERSION = '0.2.2'

# All other metadata.
# pylint: disable=C0330
METADATA = {
    'name':             NAME,
    'version':          VERSION,
    'description':      ('Pandoc filter that sets a a custom style '
                         'for the reference section header.'),
    'long_description': readme(),
    'keywords':         'pandoc reference section header',
    'classifiers':      ['Development Status :: 4 - Beta',
                 'License :: OSI Approved :: MIT License',
                 'Programming Language :: Python :: 2.7',
                 'Environment :: Console',
                 'Operating System :: OS Independent',
                 'Topic :: Text Processing :: Filters'
    ],
    'url':             'https://github.com/odkr/pandoc-refheadstyle/',
    'project_urls':     {
                'Source': 'https://github.com/odkr/pandoc-refheadstyle/',
                'Tracker': 'https://github.com/odkr/pandoc-refheadstyle/issues'
    },
    'author':           'Odin Kroeger',
    'author_email':     'tqxwcv@maskr.me',
    'license':          'MIT',
    'python_requires':  '>=2.7',
    'packages':         ['pandoc_refheadstyle'],
    'install_requires': ['panflute'],
    'zip_safe':         True
}


# Boilerplate
# ===========

if __name__ == '__main__':
    setup(**METADATA)



#!/usr/local/bin/python
# encoding=utf-8
# pylint: disable=C0103
#
# Copyright 2018, 2019 Odin Kroeger
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
# OR OTHER DEALINGS IN THE SOFTWARE.
"""Pandoc filter that sets a custom style for the reference section header."""

from panflute import Div, Header, Para, stringify

DEFAULT_STYLE = 'Bibliography Heading'


class RefHeadStyleSetter():  # pylint: disable=R0903, R1710, W0201
    """Sets a custom style for the reference section header."""

    style = DEFAULT_STYLE

    def __call__(self, elem, doc):
        r"""Set a custom style for the reference section header.

        :param panflute.Element elem: A Pandoc AST element.
        :param panflute.Doc doc: The Pandoc document that is parsed.
        :returns: A ``Div`` with the custom style set to the value of\
            ``reference-header-style`` or to 'Bibliography Heading'.\
            If *elem* isn't the reference section header, returns `None`.
        :rtype: panflute.Element or None

        If -- and only if -- the metadata field ``reference-section-title`` is
        set, sets the style of the reference section header to the value of
        the metadata field ``reference-header-style`` or to 'Bibliography
        Heading' if that metadata field has not been set or is empty.

        See :func:`set_style` on how it is determined whether a header is the
        reference section header.
        """
        try:
            if self.title:
                return set_style(elem, self.title, self.style)
        except AttributeError:
            self.title = doc.get_metadata('reference-section-title')
            style = doc.get_metadata('reference-header-style')
            if style:
                self.style = style
        return self(elem, doc)


def set_style(elem, title, style):  # pylint: disable=R1710
    r"""Change the style of the reference heading.

    :param panflute.Element elem: A Pandoc AST element.
    :param panflute.Doc doc: The Pandoc document that is parsed.
    :param str title: The title of the reference section.
    :param str style: The style of the reference section header.
    :returns: A ``Div`` with the custom style set to the value of\
        ``reference-header-style`` or to 'Bibliography Heading'.
        If *elem* isn't the reference section header, returns ``None``.
    :rtype: panflute.Element or None

    *elem* is considered the reference section header if and only if:
        1. It is an instance of Header.
        2. Its ID is 'bibliography'.
        3. Its text is the value of *title*.
    """
    if (isinstance(elem, Header) and
            elem.identifier == 'bibliography' and
            stringify(elem) == title):
        attributes = elem.attributes
        attributes['custom-style'] = style
        return Div(Para(*elem.content), identifier=elem.identifier,
                   classes=elem.classes, attributes=attributes)
